/********************************************************************\
 * readfile.h -- Parse XML file for DWI glade-to-database coupling  *
 * http://dwi.sourceforge.net                                       *
 *                                                                  *
 * This library is free software; you can redistribute it and/or    *
 * modify it under the terms of the GNU Lesser General Public       *
 * License as published by the Free Software Foundation; either     *
 * version 2.1 of the License, or (at your option) any later version.
 *                                                                  *
 * This library is distributed in the hope that it will be useful,  *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    *
 * GNU Lesser General Public License for more details.              *
 *                                                                  *
 * You should have received a copy of the GNU Lesser General Public *
 * License along with this program; if not, contact:                *
 *                                                                  *
 * Free Software Foundation           Voice:  +1-617-542-5942       *
 * 59 Temple Place - Suite 330        Fax:    +1-617-542-2652       *
 * Boston, MA  02111-1307,  USA       gnu@gnu.org                   *
\********************************************************************/
/**
 * @file readfile.h
 * @breif Parse an XML file that specifies the DWI coupling
 * @author Copyright (C) 2002,2004 Linas Vepstas <linas@linas.org>
 */

#ifndef DUI_READFILE_H
#define DUI_READFILE_H

#include "interface.h"
#include "duifield.h"
#include "duifieldmap.h"

typedef struct DuiParserElementPlugin_s DuiParserElementPlugin;
typedef struct DuiParserFieldPlugin_s DuiParserFieldPlugin;
typedef struct DuiParserTablePlugin_s DuiParserTablePlugin;
typedef struct DuiParserStructPlugin_s DuiParserStructPlugin;
typedef struct DuiParser_s DuiParser;

/** Allow user-defined XML parsers for custom markup. 
 *  When an XML element of 'eltname' is encountered, 
 *  the 'start_handler' callback will be called, and when
 *  the end element is found, the 'end_handler' will be 
 *  called.
 */
struct DuiParserElementPlugin_s 
{
	/** Name of XML element that this plugin handles */
	const char * eltname;

	/** Handler for the element, called with a pointer to this. */
	void (*start_handler) (gpointer self, const char **attrs);

	/** End handler (in case things were nested */
	void (*end_handler) (gpointer self);

	/** pointer with which the above are called. */
	gpointer user_data;
};

/** Allow user-defined XML parsers for custom markup. 
 *  When an XML field source or target element is eencountered,
 *  the 'handler' callback will be called.
 */
struct DuiParserFieldPlugin_s 
{
	/** Name of XML element that this plugin handles */
	const char * fieldname;

	/** Handler for the fieldname, called with a pointer to this. 
	 *  Handler shjould return TRUE if it handled the field, else
	 *  return false. */
	gboolean (*handler) (gpointer self, DuiField *fld,
                    const char *row_context, const char **attrs);

	/** pointer with which the above are called. */
	gpointer user_data;
};

/** Handlers for 'where' type terms. */
struct DuiParserTablePlugin_s 
{
	/** Name of plugin, for debugging purposes only */
	const char * plugin_name;
	
	/** Handler for the fieldname, called with a pointer to this. */
	const char * (*handler) (gpointer self,  DuiFieldMap *map, const char **attrs);

	/** pointer with which the above are called. */
	gpointer user_data;
};

/** Get notified when certain strucures get created. 
 *  XXX long term, this should be replaced by a "real" notifier
 *  system for create/destroy/modify events.
 */
struct DuiParserStructPlugin_s 
{
	void (*action_created) (gpointer self, DuiAction *act);
	void (*report_created) (gpointer self, DuiReport *rpt);

	/** pointer with which the above are called. */
	gpointer user_data;
};

/** Create a new instance of the parser.  The parser will use the 
 * indicated 'interface' for storing top-level data */
DuiParser *dui_parser_new (DuiInterface *);

void dui_parser_destroy (DuiParser *);

DuiInterface * dui_parser_get_interface (DuiParser *);

/** Read in and parse the indicated filename.  It had better contain a 
 *  valid dui description!
 */
void dui_parse (DuiParser *, const char * xml_filename);

/** Register a new plugin that will be used during parsing */
void dui_parser_register_element_plugin (DuiParser *, DuiParserElementPlugin *);

/** Register a new plugin that will handle fields during parsing */
void dui_parser_register_field_plugin (DuiParser *, DuiParserFieldPlugin *);

/** Register a new plugin that will handle structs during parsing */
void dui_parser_register_struct_plugin (DuiParser *, DuiParserStructPlugin *);

#endif /* DUI_READFILE_H */

