/*
 * FILE:
 * view-entry.c
 *
 * FUNCTION:
 * Example of integration between a pre-existing GTK application, and DWI.
 *  
 * This is a fairly simple example showing how to run an arbitary SQL query,
 * and have it be displayed in a DWI report.  Most of the complexity of this
 * example surrounds the need to obtain a handle to the database so that a
 * query can be run.
 *
 * HISTORY:
 * Linas Vepstas July 2002
 */

#include "config.h"

#include <stdio.h>

#include <glib-object.h>
#include <gnome.h>
#include <glade/glade.h>

#include "database.h"
#include "interface.h"
#include "report.h"
#include "readfile.h"
#include "show-message.h"

/* some globals -- makes this example program shorter */
DuiInterface *di = NULL;

/* =================================================================== */

void
button_clicked_cb (GtkButton *button, GtkWidget *w)
{
	char query[300];
	DuiDatabase *db;
	DuiDBConnection *db_conn;
	DuiReport *report;
	DuiDBRecordSet *recs = NULL;
	GtkEntry *ew = GTK_ENTRY (w);
	int record_id = 0;

	/* Provide fedback to the user about what they entered. */
	g_message ("Button was clicked; entry box contains \'%s\'\n", gtk_entry_get_text(ew));

	/* Make sure that they entered a valid number */
	record_id = atoi (gtk_entry_get_text(ew));
	if (0 == record_id)
	{
		g_message ("The entry box should contain the numeric record id\n"
		           "of a record in the \'Notes\' database table");
		return;
	}

	/* Find a database connection.  In this example, we use the connection
	 * that DWI set up for us; although we could also use dui_database_new()
	 * if we wanted to.
	 */
	db = dui_interface_find_database_by_name (di, "my db object");
	db_conn = dui_database_do_realize (db);

	/* Create an 'arbitary' query.   Of course, the DWI report that will
	 * show the results of this query must be able to understand the 
	 * record (row) that was returned. 
	 */
	sprintf (query, "SELECT * FROM notes WHERE id=\'%d\';", record_id);
	recs = dui_connection_exec(db_conn, query);

	/* Show the returned record.  We do this in a named report.  To see the
	 * DWI definition of this report, grep for 'name="record viewer"'
	 * in the "testbasic.dui" file.
	 */
	report = dui_interface_find_report_by_name (di, "record viewer");
	dui_report_set_last_action (report, NULL);
	dui_report_show_data (report, recs);
	dui_recordset_free (recs);
}

/* =================================================================== */
/* create a window with a button (by hand) */

void
make_window(void)
{
	GtkWidget *main_w, *vbox_w, *button_w, *entry_w;
  
  /* Generic window setup --
   * The next block is plain-old 'typical' GTK setup code */
  main_w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW(main_w), "Integration Demo");
  gtk_widget_set_usize (main_w, 150, 150);
  // gtk_container_border_width(GTK_CONTAINER(main_w),0);

  g_signal_connect (G_OBJECT (main_w), "destroy",
 		     G_CALLBACK (gtk_main_quit), NULL);

  vbox_w = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER(main_w), vbox_w);
  gtk_widget_show (vbox_w);

  entry_w = gtk_entry_new();
  gtk_container_add (GTK_CONTAINER(vbox_w), entry_w);
  gtk_widget_show (entry_w);
  
  button_w = gtk_button_new_with_label ("Press Me");
  gtk_container_add (GTK_CONTAINER(vbox_w), button_w);
  gtk_widget_show (button_w);

  g_signal_connect (G_OBJECT(button_w), "clicked", 
           G_CALLBACK(button_clicked_cb), entry_w);

  gtk_widget_show (main_w);
}

/* =================================================================== */

int
main (int argc, char *argv[])
{
	/* generic gnome/gtk initialization */
	g_type_init ();

	gnome_init ("main", VERSION, argc, argv);
	glade_gnome_init();

	/* Use GUI pops to report error messages */
#define ERR_ONLY	 G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING 
#define DBG_MSG	 ERR_ONLY | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG 
	g_log_set_handler (G_LOG_DOMAIN, DBG_MSG, dui_show_message, NULL);
	
	/* DUI initialization */
	di = dui_interface_new();
	DuiParser *par = dui_parser_new(di);
	dwi_gtk_parser_new (par);
	dui_parse (par, "testbasic.dui");
	dui_parser_destroy (par);
	dui_interface_realize (di, TRUE);

	/* create a GTK button, hook it to a DWI report */
	make_window();

	gtk_main ();

	dui_interface_destroy(di);
	printf ("Bye! Please Come Again!\n");

	return 0;
}

/* ========================== END OF FILE ============================ */
