/*
 * FILE:
 * fall-object.h
 *
 * FUNCTION:
 * Demonstrates a basic GObject, using the glib-gobject interface.
 * This file implements a basic GObject, which will be used to
 * demonstrate how to use gobjects with DWI.
 *
 * Unfortunately, this file mostly demonstrates how verbose
 * the glib-gobject infrastructure is.  There's a lot of 
 * boiler-plate in here, a lot of generic code.
 *
 * I strongly suggest that you use "gob" to automatically
 * create your gobjects.  See http://www.5z.com/jirka/gob.html
 * for details; alternately, 'apt-get install gob2'
 * For an example usage of gob with dwi, see the "bike-calc" directory.
 *
 * The "falling object" is a gobject that has two inputs and 
 * one output.  The output is computed "on the fly" based on 
 * the inputs.  Both the inputs and the output are accessed 
 * through gobject "properties".
 *
 * HISTORY:
 * Linas Vepstas <linas@linas.org> March 2004
 */

#include <glib.h>
#include <glib-object.h>
#include <stdio.h>
#include <stdlib.h>

#include "fall-object.h"

struct FallingObject_s
{
	GObject gobject;

	int input_a;
	int input_b;
	int output_result;
};

/* properties */
enum
{
	FALLIN_A=1,
	FALLIN_B,
	FALLOUT,
};

typedef struct FallingClass_s
{
	GObjectClass parent_class;
} FallingClass;

/* =================================================================== */

static void
fall_get_property (GObject *object, guint property_id,
                   GValue *value, GParamSpec  *pspec)
{
	GValue val={0};
	int ival=0;
	Falling *fall = FALL(object);

	switch (property_id)
	{
		case FALLIN_A:    
			ival = fall->input_a; 
			printf ("Falling Object Requested Input A (value=%d)\n", ival);
			break;
		case FALLIN_B:    
			ival = fall->input_b; 
			printf ("Falling Object Requested Input B (value=%d)\n", ival);
			break;
		case FALLOUT:     
			fall->output_result = fall->input_a + fall->input_b;
			ival = fall->output_result; 
			printf ("Falling Object Requested Output (value=%d)\n", ival);
			break;
	}
	char buff[40];
	sprintf (buff, "%d", ival);
	g_value_init (&val, G_TYPE_STRING);
	g_value_set_string (&val, buff);
	g_value_copy (&val, value);
}

static void
fall_set_property (GObject *object, guint property_id,
                   const GValue *value, GParamSpec  *pspec)
{
	Falling *fall = FALL(object);

	/* DWI passes all values as strings, thus we will receive the
	 * value as a string, and we have to convert it ourselves to an int.
	 */
	const char * str = g_value_get_string (value);
	int val = atoi (str);
	
	switch (property_id)
	{
		case FALLIN_A:
			fall->input_a = val;
			printf ("Falling Object Input A was set to %d\n", fall->input_a);
			break;
		case FALLIN_B:
			fall->input_b = val;
			printf ("Falling Object Input B was set to %d\n", fall->input_b);
			break;
	}
}

/* =================================================================== */

#define INSTALL_RW(PROP,NAME,DESC) {                             \
   GParamSpec *pspec;                                            \
   pspec = g_param_spec_string (NAME, NULL,                      \
               DESC, NULL, G_PARAM_READWRITE);                   \
   g_object_class_install_property (goc, PROP, pspec);           \
}

#define INSTALL_RO(PROP,NAME,DESC) {                             \
   GParamSpec *pspec;                                            \
   pspec = g_param_spec_string (NAME, NULL,                      \
               DESC, NULL, G_PARAM_READABLE);                    \
   g_object_class_install_property (goc, PROP, pspec);           \
}

static void
fall_class_init (FallingClass *fallclass, gpointer class_data)
{
	GObjectClass *goc;

	fallclass->parent_class.set_property = fall_set_property;
	fallclass->parent_class.get_property = fall_get_property;

	goc = G_OBJECT_CLASS (fallclass);

	INSTALL_RW (FALLIN_A, "input_a",  "First Input to Calculation");
	INSTALL_RW (FALLIN_B, "input_b",  "Second Input to Calculation");
	INSTALL_RO (FALLOUT,  "output",   "Output of Calculation");
}

/* =================================================================== */

static void
fall_init (Falling *fall, FallingClass *falldclass)
{
	fall->input_a = 0;
	fall->input_b = 0;
	fall->output_result = 0;
}

/* =================================================================== */

GType
fall_get_type (void)
{
	static GType fall_type = 0;

	if (!fall_type)
	{
		GTypeInfo fall_info =
		{
			sizeof (FallingClass),
			NULL,
			NULL,
			(GClassInitFunc) fall_class_init,
			NULL, NULL,
			sizeof (Falling),
			0,
			(GInstanceInitFunc) fall_init,
			NULL
		};

		fall_type = g_type_register_static (G_TYPE_OBJECT,
		     "Falling", &fall_info, 0);
	}
	return fall_type;
}

/* =================================================================== */

Falling *
fall_new (void)
{
	Falling *fall;

	fall = g_object_new (FALL_TYPE, NULL);
	return fall;
}

/* =================================================================== */
