
/** @file my-object.c
 *  @breif Example definition of a QOF Entity.
 *  @author Copyright (c) 2003,2004 Linas Vepstas <linas@linas.org>
 *
 *  This example shows how to derive from the QOF Entity class.
 *
 * Please make note of how the 'book' is used.  A 'book' can serve
 * as a collection of collections.  In the example below, the book
 * will be used to hold all of the instances of 'my object' so that
 * these can be found later.  Note that the book doesn't need to 
 * exclusively hold only instances of 'my object', it can hold 
 * collections of other types as well. 
 *
 * The above ability is one reason why 'book' holds such a central
 * place in the QOF system.  Note also: when the query is performed
 * over all instances of 'my object', its not really over *all* 
 * instances; its only over those instances stored in the given book.
 * Thus, the book is a kind of dataset, and by using books, one
 * can have multiple disjoint datasets.
 */

#include <glib.h>
#include <qof/qof.h>
#include "my-entity.h"

/* ===================================================== */


MyEnt *
my_ent_new (QofBook *book)
{
	MyEnt *m = g_new0 (MyEnt,1);

	QofCollection *coll = qof_book_get_collection (book, MYENT_ID);
	qof_entity_init (&m->ent, MYENT_ID, coll);

	return m;
}

/* Generic object getters --------------------------------- */
int
my_ent_get_a (MyEnt *m)
{
	return m->a;
}

int
my_ent_get_b (MyEnt *m)
{
	return m->b;
}

const char *
my_ent_get_memo (MyEnt *m)
{
	return m->memo;
}

/* Generic object setters --------------------------------- */

void
my_ent_set_a (MyEnt *m, int x)
{
	m->a = x;
}

void
my_ent_set_b (MyEnt *m, int y)
{
	m->b = y;
}

void
my_ent_set_memo (MyEnt *m, const char * str)
{
	if (m->memo) g_free (m->memo);
	m->memo = g_strdup (str);
}

/* ===================================================== */
/* Provide a default mechanism to sort MyEnt 
 * This is neeeded so that query results can be returned in
 * some 'reasonable' order.  If you don't want to sort,
 * just have this function always return 0.
 */ 
int
my_ent_order (MyEnt *a, MyEnt *b)
{
	if ( (a) && !(b) ) return -1;
	if ( !(a) && (b) ) return +1;
	if ( !(a) && !(b) ) return 0;

	if ((a)->a > (b)->a) return +1;
	if ((a)->a == (b)->a) return 0;
	return -1;
}

const char * 
my_ent_print (MyEnt *m)
{
	static char buff[200];
	sprintf (buff, "a=%d b=%d memo=%s\n", m->a, m->b, m->memo);
	return buff;
}

/* ===================================================== */
/* Provide infrastructure to register my object with QOF */

static QofObject myent_object_def = 
{
	interface_version: QOF_OBJECT_VERSION,
	e_type:            MYENT_ID,
	type_label:        "My Blinking Object",
	create:            my_ent_new,
	book_begin:        NULL,
	book_end:          NULL,
	is_dirty:          NULL,
	mark_clean:        NULL,
	foreach:           qof_collection_foreach,
	printable:         my_ent_print,
};

gboolean my_ent_register (void)
{
	/* Associate an ASCII name to each getter, as well as the return type */
	static QofParam params[] = {
	  { MYENT_A,	  QOF_TYPE_INT32, (QofAccessFunc)my_ent_get_a, 
	                                 (QofSetterFunc)my_ent_set_a },
	  { MYENT_B,	  QOF_TYPE_INT32, (QofAccessFunc)my_ent_get_b, 
	                                 (QofSetterFunc)my_ent_set_b },
	  { MYENT_MEMO, QOF_TYPE_STRING, (QofAccessFunc)my_ent_get_memo, 
	                                 (QofSetterFunc)my_ent_set_memo },
	  { QOF_PARAM_GUID, QOF_TYPE_GUID,
	                         (QofAccessFunc)qof_entity_get_guid, NULL },

	  { NULL },
	};

	qof_class_register (MYENT_ID, (QofSortFunc)my_ent_order, params);
	return qof_object_register (&myent_object_def);
}

/* =================== END OF FILE ===================== */
